@@ -17,8 +17,6 @@
 #ifndef LIBXLOG_H
 #define LIBXLOG_H
 
-#include <xfs/libxfs.h>
-
 /*
  * define the userlevel xlog_t to be the subset of the kernel's
  * xlog_t that we actually need to get our work done, avoiding
@@ -30,7 +28,6 @@ struct xlog {
 	xfs_mount_t	*l_mp;	        /* mount point */
 	struct xfs_buftarg *l_dev;	        /* dev_t of log */
 	xfs_daddr_t	l_logBBstart;   /* start block of log */
-	int		l_logsize;      /* size of log in bytes */
 	int		l_logBBsize;    /* size of log in 512 byte chunks */
 	int		l_curr_cycle;   /* Cycle number of log writes */
 	int		l_prev_cycle;   /* Cycle # b4 last block increment */
@@ -47,10 +44,12 @@ struct xlog {
 	int		l_sectBBsize;   /* size of log sector in 512 byte chunks */
 };
 
-#include <xfs/xfs_log_recover.h>
+#include "xfs_log_recover.h"
 
 /*
  * macros mapping kernel code to user code
+ *
+ * XXX: this is duplicated stuff - should be shared with libxfs.
  */
 #ifndef EFSCORRUPTED
 #define EFSCORRUPTED			 990
@@ -65,6 +64,12 @@ struct xlog {
 #define XFS_CORRUPTION_ERROR(e,l,mp,m)	((void) 0)
 #define XFS_MOUNT_WAS_CLEAN		0x1
 #define unlikely(x)			(x)
+#define xfs_alert(mp,fmt,args...)	cmn_err(CE_ALERT,fmt, ## args)
+#define xfs_warn(mp,fmt,args...)	cmn_err(CE_WARN,fmt, ## args)
+#define xfs_hex_dump(d,n)		((void) 0)
+#define __round_mask(x, y) ((__typeof__(x))((y)-1))
+#define round_up(x, y) ((((x)-1) | __round_mask(x, y))+1)
+#define round_down(x, y) ((x) & ~__round_mask(x, y))
 
 extern void xlog_warn(char *fmt,...);
 extern void xlog_exit(char *fmt,...);
@@ -83,7 +88,7 @@ extern int xlog_is_dirty(xfs_mount_t *mp, libxfs_init_t *x, int verbose);
 extern struct xfs_buf *xlog_get_bp(struct xlog *, int);
 extern void	xlog_put_bp(struct xfs_buf *);
 extern int	xlog_bread(struct xlog *log, xfs_daddr_t blk_no, int nbblks,
-				xfs_buf_t *bp, xfs_caddr_t *offset);
+				xfs_buf_t *bp, char **offset);
 extern int	xlog_bread_noalign(struct xlog *log, xfs_daddr_t blk_no,
 				int nbblks, xfs_buf_t *bp);
 
@@ -96,7 +101,7 @@ extern int	xlog_find_tail(struct xlog *log, xfs_daddr_t *head_blk,
 
 extern int	xlog_test_footer(struct xlog *log);
 extern int	xlog_recover(struct xlog *log, int readonly);
-extern void	xlog_recover_print_data(xfs_caddr_t p, int len);
+extern void	xlog_recover_print_data(char *p, int len);
 extern void	xlog_recover_print_logitem(xlog_recover_item_t *item);
 extern void	xlog_recover_print_trans_head(xlog_recover_t *tr);
 extern int	xlog_print_find_oldest(struct xlog *log, xfs_daddr_t *last_blk);
