@@ -17,9 +17,10 @@
  */
 
 #include <libgen.h>
-#include <xfs/xfs.h>
-#include <xfs/handle.h>
-#include <xfs/parent.h>
+#include "platform_defs.h"
+#include "xfs.h"
+#include "handle.h"
+#include "parent.h"
 
 /* just pick a value we know is more than big enough */
 #define	MAXHANSIZ	64
@@ -97,6 +98,7 @@ path_to_fshandle(
 		/* new filesystem. add it to the cache */
 		fdhp = malloc(sizeof(struct fdhash));
 		if (fdhp == NULL) {
+			free(*fshanp);
 			close(fd);
 			errno = ENOMEM;
 			return -1;
