@@ -16,10 +16,23 @@
  * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs.h>
-#include <time.h>
-#include <stdio.h>
-#include <stdarg.h>
+#include "libxfs_priv.h"
+#include "init.h"
+#include "xfs_fs.h"
+#include "xfs_shared.h"
+#include "xfs_format.h"
+#include "xfs_log_format.h"
+#include "xfs_trans_resv.h"
+#include "xfs_mount.h"
+#include "xfs_inode_buf.h"
+#include "xfs_inode_fork.h"
+#include "xfs_inode.h"
+#include "xfs_trans.h"
+#include "xfs_bmap.h"
+#include "xfs_bmap_btree.h"
+#include "xfs_trans_space.h"
+#include "xfs_ialloc.h"
+#include "xfs_alloc.h"
 
 /*
  * Calculate the worst case log unit reservation for a given superblock
@@ -217,13 +230,12 @@ libxfs_ialloc(
 	xfs_trans_ichgtime(tp, ip, XFS_ICHGTIME_CHG | XFS_ICHGTIME_MOD);
 
 	/*
-	 * If the superblock version is up to where we support new format
-	 * inodes and this is currently an old format inode, then change
-	 * the inode version number now.  This way we only do the conversion
-	 * here rather than here and in the flush/logging code.
+	 * We only support filesystems that understand v2 format inodes. So if
+	 * this is currently an old format inode, then change the inode version
+	 * number now.  This way we only do the conversion here rather than here
+	 * and in the flush/logging code.
 	 */
-	if (xfs_sb_version_hasnlink(&tp->t_mountp->m_sb) &&
-	    ip->i_d.di_version == 1) {
+	if (ip->i_d.di_version == 1) {
 		ip->i_d.di_version = 2;
 		/*
 		 * old link count, projid_lo/hi field, pad field
@@ -250,7 +262,7 @@ libxfs_ialloc(
 
 	if (ip->i_d.di_version == 3) {
 		ASSERT(ip->i_d.di_ino == ino);
-		ASSERT(uuid_equal(&ip->i_d.di_uuid, &mp->m_sb.sb_uuid));
+		ASSERT(uuid_equal(&ip->i_d.di_uuid, &mp->m_sb.sb_meta_uuid));
 		ip->i_d.di_crc = 0;
 		ip->i_d.di_changecount = 1;
 		ip->i_d.di_lsn = 0;
@@ -311,6 +323,14 @@ libxfs_ialloc(
 	ip->i_d.di_aformat = XFS_DINODE_FMT_EXTENTS;
 	ip->i_d.di_anextents = 0;
 
+	/*
+	 * set up the inode ops structure that the libxfs code relies on
+	 */
+	if (S_ISDIR(ip->i_d.di_mode))
+		ip->d_ops = ip->i_mount->m_dir_inode_ops;
+	else
+		ip->d_ops = ip->i_mount->m_nondir_inode_ops;
+
 	/*
 	 * Log the new values stuffed into the inode.
 	 */
@@ -399,12 +419,13 @@ libxfs_iflush_int(xfs_inode_t *ip, xfs_buf_t *bp)
 	ASSERT(XFS_BUF_FSPRIVATE(bp, void *) != NULL);
 	ASSERT(ip->i_d.di_format != XFS_DINODE_FMT_BTREE ||
 		ip->i_d.di_nextents > ip->i_df.if_ext_max);
+	ASSERT(ip->i_d.di_version > 1);
 
 	iip = ip->i_itemp;
 	mp = ip->i_mount;
 
 	/* set *dip = inode's place in the buffer */
-	dip = (xfs_dinode_t *)xfs_buf_offset(bp, ip->i_imap.im_boffset);
+	dip = xfs_buf_offset(bp, ip->i_imap.im_boffset);
 
 	ASSERT(ip->i_d.di_magic == XFS_DINODE_MAGIC);
 	if ((ip->i_d.di_mode & S_IFMT) == S_IFREG) {
@@ -431,40 +452,9 @@ libxfs_iflush_int(xfs_inode_t *ip, xfs_buf_t *bp)
 	 */
 	xfs_dinode_to_disk(dip, &ip->i_d);
 
-	/*
-	 * If this is really an old format inode and the superblock version
-	 * has not been updated to support only new format inodes, then
-	 * convert back to the old inode format.  If the superblock version
-	 * has been updated, then make the conversion permanent.
-	 */
-	ASSERT(ip->i_d.di_version == 1 ||
-		xfs_sb_version_hasnlink(&mp->m_sb));
-	if (ip->i_d.di_version == 1) {
-		if (!xfs_sb_version_hasnlink(&mp->m_sb)) {
-			/*
-			 * Convert it back.
-			 */
-			ASSERT(ip->i_d.di_nlink <= XFS_MAXLINK_1);
-			dip->di_onlink = cpu_to_be16(ip->i_d.di_nlink);
-		} else {
-			/*
-			 * The superblock version has already been bumped,
-			 * so just make the conversion to the new inode
-			 * format permanent.
-			 */
-			ip->i_d.di_version = 2;
-			dip->di_version =  2;
-			ip->i_d.di_onlink = 0;
-			dip->di_onlink = 0;
-			memset(&(ip->i_d.di_pad[0]), 0, sizeof(ip->i_d.di_pad));
-			memset(&(dip->di_pad[0]), 0, sizeof(dip->di_pad));
-			ASSERT(xfs_get_projid(&ip->i_d) == 0);
-		}
-	}
-
-	xfs_iflush_fork(ip, dip, iip, XFS_DATA_FORK, bp);
+	xfs_iflush_fork(ip, dip, iip, XFS_DATA_FORK);
 	if (XFS_IFORK_Q(ip)) 
-		xfs_iflush_fork(ip, dip, iip, XFS_ATTR_FORK, bp);
+		xfs_iflush_fork(ip, dip, iip, XFS_ATTR_FORK);
 
 	/* update the lsn in the on disk inode if required */
 	if (ip->i_d.di_version == 3)
@@ -476,35 +466,26 @@ libxfs_iflush_int(xfs_inode_t *ip, xfs_buf_t *bp)
 	return 0;
 }
 
-/*
- * Utility routine common used to apply a delta to a field in the
- * in-core superblock.
- * Switch on the field indicated and apply the delta to that field.
- * Fields are not allowed to dip below zero, so if the delta would
- * do this do not apply it and return EINVAL.
- *
- * Originally derived from xfs_mod_incore_sb_unlocked().
- */
 int
 libxfs_mod_incore_sb(
-	xfs_mount_t	*mp,
-	xfs_sb_field_t	field,
+	struct xfs_mount *mp,
+	int		field,
 	int64_t		delta,
 	int		rsvd)
 {
 	long long	lcounter;	/* long counter for 64 bit fields */
 
 	switch (field) {
-	case XFS_SBS_FDBLOCKS:
+	case XFS_TRANS_SB_FDBLOCKS:
 		lcounter = (long long)mp->m_sb.sb_fdblocks;
 		lcounter += delta;
 		if (lcounter < 0)
-			return XFS_ERROR(ENOSPC);
+			return -ENOSPC;
 		mp->m_sb.sb_fdblocks = lcounter;
 		return 0;
 	default:
 		ASSERT(0);
-		return XFS_ERROR(EINVAL);
+		return -EINVAL;
 	}
 }
 
@@ -564,7 +545,7 @@ libxfs_alloc_file_space(
 	int		error;
 
 	if (len <= 0)
-		return EINVAL;
+		return -EINVAL;
 
 	count = len;
 	error = 0;
@@ -590,8 +571,8 @@ libxfs_alloc_file_space(
 			/*
 			 * Free the transaction structure.
 			 */
-			ASSERT(error == ENOSPC);
-			xfs_trans_cancel(tp, 0);
+			ASSERT(error == -ENOSPC);
+			xfs_trans_cancel(tp);
 			break;
 		}
 		xfs_trans_ijoin(tp, ip, 0);
@@ -609,13 +590,13 @@ libxfs_alloc_file_space(
 		if (error)
 			goto error0;
 
-		error = xfs_trans_commit(tp, 0);
+		error = xfs_trans_commit(tp);
 		if (error)
 			break;
 
 		allocated_fsb = imapp->br_blockcount;
 		if (reccount == 0)
-			return ENOSPC;
+			return -ENOSPC;
 
 		startoffset_fsb += allocated_fsb;
 		allocatesize_fsb -= allocated_fsb;
@@ -624,7 +605,7 @@ libxfs_alloc_file_space(
 
 error0:	/* Cancel bmap, cancel trans */
 	xfs_bmap_cancel(&free_list);
-	xfs_trans_cancel(tp, 0);
+	xfs_trans_cancel(tp);
 	return error;
 }
 
@@ -660,7 +641,6 @@ libxfs_inode_alloc(
 {
 	xfs_buf_t	*ialloc_context;
 	xfs_inode_t	*ip;
-	xfs_trans_t	*ntp;
 	int		error;
 
 	ialloc_context = (xfs_buf_t *)0;
@@ -672,23 +652,16 @@ libxfs_inode_alloc(
 	}
 	if (!ialloc_context && !ip) {
 		*ipp = NULL;
-		return XFS_ERROR(ENOSPC);
+		return -ENOSPC;
 	}
 
 	if (ialloc_context) {
-		struct xfs_trans_res	tres;
 
 		xfs_trans_bhold(*tp, ialloc_context);
-		tres.tr_logres = (*tp)->t_log_res;
-		tres.tr_logcount = (*tp)->t_log_count;
 
-		ntp = xfs_trans_dup(*tp);
-		xfs_trans_commit(*tp, 0);
-		*tp = ntp;
-		tres.tr_logflags = XFS_TRANS_PERM_LOG_RES;
-		error = xfs_trans_reserve(*tp, &tres, 0, 0);
+		error = xfs_trans_roll(tp, NULL);
 		if (error) {
-			fprintf(stderr, _("%s: cannot reserve space: %s\n"),
+			fprintf(stderr, _("%s: cannot duplicate transaction: %s\n"),
 				progname, strerror(error));
 			exit(1);
 		}
@@ -696,7 +669,7 @@ libxfs_inode_alloc(
 		error = libxfs_ialloc(*tp, pip, mode, nlink, rdev, cr,
 				   fsx, 1, &ialloc_context, &ip);
 		if (!ip)
-			error = ENOSPC;
+			error = -ENOSPC;
 		if (error)
 			return error;
 	}
@@ -753,6 +726,6 @@ xfs_verifier_error(
 	struct xfs_buf		*bp)
 {
 	xfs_alert(NULL, "Metadata %s detected at block 0x%llx/0x%x",
-		  bp->b_error == EFSBADCRC ? "CRC error" : "corruption",
+		  bp->b_error == -EFSBADCRC ? "CRC error" : "corruption",
 		  bp->b_bn, BBTOB(bp->b_length));
 }
