@@ -16,7 +16,7 @@
  * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs/command.h>
+#include "command.h"
 #include "init.h"
 #include "quota.h"
 
@@ -122,10 +122,13 @@ state_qfilestat(
 		mount->fs_dir, mount->fs_name);
 	fprintf(fp, _("  Accounting: %s\n"), accounting ? _("ON") : _("OFF"));
 	fprintf(fp, _("  Enforcement: %s\n"), enforcing ? _("ON") : _("OFF"));
-	fprintf(fp, _("  Inode: #%llu (%llu blocks, %lu extents)\n"),
-		(unsigned long long)qfs->qfs_ino,
-		(unsigned long long)qfs->qfs_nblks,
-		(unsigned long)qfs->qfs_nextents);
+	if (qfs->qfs_ino != (__u64) -1)
+		fprintf(fp, _("  Inode: #%llu (%llu blocks, %lu extents)\n"),
+			(unsigned long long)qfs->qfs_ino,
+			(unsigned long long)qfs->qfs_nblks,
+			(unsigned long)qfs->qfs_nextents);
+	else
+		fprintf(fp, _("  Inode: N/A\n"));
 }
 
 static void
@@ -253,6 +256,7 @@ enable_enforcement(
 
 	mount = fs_table_lookup(dir, FS_MOUNT_POINT);
 	if (!mount) {
+		exitcode = 1;
 		fprintf(stderr, "%s: unknown mount point %s\n", progname, dir);
 		return;
 	}
@@ -274,6 +278,7 @@ disable_enforcement(
 
 	mount = fs_table_lookup(dir, FS_MOUNT_POINT);
 	if (!mount) {
+		exitcode = 1;
 		fprintf(stderr, "%s: unknown mount point %s\n", progname, dir);
 		return;
 	}
@@ -295,6 +300,7 @@ quotaoff(
 
 	mount = fs_table_lookup(dir, FS_MOUNT_POINT);
 	if (!mount) {
+		exitcode = 1;
 		fprintf(stderr, "%s: unknown mount point %s\n", progname, dir);
 		return;
 	}
@@ -327,6 +333,7 @@ remove_extents(
 
 	mount = fs_table_lookup(dir, FS_MOUNT_POINT);
 	if (!mount) {
+		exitcode = 1;
 		fprintf(stderr, "%s: unknown mount point %s\n", progname, dir);
 		return;
 	}
@@ -513,7 +520,7 @@ remove_f(
 void
 state_init(void)
 {
-	off_cmd.name = _("off");
+	off_cmd.name = "off";
 	off_cmd.cfunc = off_f;
 	off_cmd.argmin = 0;
 	off_cmd.argmax = -1;
@@ -521,7 +528,7 @@ state_init(void)
 	off_cmd.oneline = _("permanently switch quota off for a path");
 	off_cmd.help = off_help;
 
-	state_cmd.name = _("state");
+	state_cmd.name = "state";
 	state_cmd.cfunc = state_f;
 	state_cmd.argmin = 0;
 	state_cmd.argmax = -1;
@@ -529,7 +536,7 @@ state_init(void)
 	state_cmd.oneline = _("get overall quota state information");
 	state_cmd.help = state_help;
 
-	enable_cmd.name = _("enable");
+	enable_cmd.name = "enable";
 	enable_cmd.cfunc = enable_f;
 	enable_cmd.argmin = 0;
 	enable_cmd.argmax = -1;
@@ -537,7 +544,7 @@ state_init(void)
 	enable_cmd.oneline = _("enable quota enforcement");
 	enable_cmd.help = enable_help;
 
-	disable_cmd.name = _("disable");
+	disable_cmd.name = "disable";
 	disable_cmd.cfunc = disable_f;
 	disable_cmd.argmin = 0;
 	disable_cmd.argmax = -1;
@@ -545,7 +552,7 @@ state_init(void)
 	disable_cmd.oneline = _("disable quota enforcement");
 	disable_cmd.help = disable_help;
 
-	remove_cmd.name = _("remove");
+	remove_cmd.name = "remove";
 	remove_cmd.cfunc = remove_f;
 	remove_cmd.argmin = 0;
 	remove_cmd.argmax = -1;
