@@ -1,4 +1,4 @@
-#include <libxfs.h>
+#include "libxfs.h"
 #include <pthread.h>
 #include "avl.h"
 #include "btree.h"
@@ -165,8 +165,8 @@ pf_read_bmbt_reclist(
 {
 	int			i;
 	xfs_bmbt_irec_t		irec;
-	xfs_dfilblks_t		cp = 0;		/* prev count */
-	xfs_dfiloff_t		op = 0;		/* prev offset */
+	xfs_filblks_t		cp = 0;		/* prev count */
+	xfs_fileoff_t		op = 0;		/* prev offset */
 #define MAP_ARRAY_SZ 4
 	struct xfs_buf_map	map_array[MAP_ARRAY_SZ];
 	struct xfs_buf_map	*map = map_array;
@@ -189,7 +189,7 @@ pf_read_bmbt_reclist(
 			goto out_free;
 
 		if (!args->dirs_only && ((irec.br_startoff +
-				irec.br_blockcount) >= mp->m_dirfreeblk))
+				irec.br_blockcount) >= mp->m_dir_geo->freeblk))
 			break;	/* only Phase 6 reads the free blocks */
 
 		op = irec.br_startoff;
@@ -200,8 +200,8 @@ pf_read_bmbt_reclist(
 
 			pftrace("queuing dir extent in AG %d", args->agno);
 
-			if (len + irec.br_blockcount >= mp->m_dirblkfsbs)
-				bm_len = mp->m_dirblkfsbs - len;
+			if (len + irec.br_blockcount >= mp->m_dir_geo->fsbcount)
+				bm_len = mp->m_dir_geo->fsbcount - len;
 			else
 				bm_len = irec.br_blockcount;
 			len += bm_len;
@@ -211,7 +211,7 @@ pf_read_bmbt_reclist(
 			map[nmaps].bm_len = XFS_FSB_TO_BB(mp, bm_len);
 			nmaps++;
 
-			if (len == mp->m_dirblkfsbs) {
+			if (len == mp->m_dir_geo->fsbcount) {
 				pf_queue_io(args, map, nmaps, B_DIR_META);
 				len = 0;
 				nmaps = 0;
@@ -257,7 +257,7 @@ out_free:
 
 static int
 pf_scan_lbtree(
-	xfs_dfsbno_t		dbno,
+	xfs_fsblock_t		dbno,
 	int			level,
 	int			isadir,
 	prefetch_args_t		*args,
@@ -276,6 +276,18 @@ pf_scan_lbtree(
 
 	XFS_BUF_SET_PRIORITY(bp, isadir ? B_DIR_BMAP : B_BMAP);
 
+	/*
+	 * If the verifier flagged a problem with the buffer, we can't trust
+	 * its contents for the purposes of reading ahead.  Stop prefetching
+	 * the tree and mark the buffer unchecked so that the next read of the
+	 * buffer will retain the error status and be acted upon appropriately.
+	 */
+	if (bp->b_error) {
+		bp->b_flags |= LIBXFS_B_UNCHECKED;
+		libxfs_putbuf(bp);
+		return 0;
+	}
+
 	rc = (*func)(XFS_BUF_TO_BLOCK(bp), level - 1, isadir, args);
 
 	libxfs_putbuf(bp);
@@ -293,7 +305,7 @@ pf_scanfunc_bmap(
 	xfs_bmbt_ptr_t		*pp;
 	int 			numrecs;
 	int			i;
-	xfs_dfsbno_t		dbno;
+	xfs_fsblock_t		dbno;
 
 	/*
 	 * do some validation on the block contents
@@ -340,7 +352,7 @@ pf_read_btinode(
 	int			level;
 	int			numrecs;
 	int			dsize;
-	xfs_dfsbno_t		dbno;
+	xfs_fsblock_t		dbno;
 
 	dib = (xfs_bmdr_block_t *)XFS_DFORK_DPTR(dino);
 
@@ -357,7 +369,7 @@ pf_read_btinode(
 		return;
 
 	dsize = XFS_DFORK_DSIZE(dino, mp);
-	pp = XFS_BMDR_PTR_ADDR(dib, 1, xfs_bmdr_maxrecs(mp, dsize, 0));
+	pp = XFS_BMDR_PTR_ADDR(dib, 1, xfs_bmdr_maxrecs(dsize, 0));
 
 	for (i = 0; i < numrecs; i++) {
 		dbno = be64_to_cpu(pp[i]);
@@ -419,8 +431,7 @@ pf_read_inode_dirs(
 		if (be16_to_cpu(dino->di_magic) != XFS_DINODE_MAGIC)
 			continue;
 
-		if (!XFS_DINODE_GOOD_VERSION(dino->di_version) ||
-				(!fs_inode_nlink && dino->di_version > 1))
+		if (!xfs_dinode_good_version(mp, dino->di_version))
 			continue;
 
 		if (be64_to_cpu(dino->di_size) <= XFS_DFORK_DSIZE(dino, mp))
@@ -680,8 +691,9 @@ pf_queuing_worker(
 	xfs_agblock_t		bno;
 	int			i;
 	int			err;
+	uint64_t		sparse;
 
-	blks_per_cluster =  XFS_INODE_CLUSTER_SIZE(mp) >> mp->m_sb.sb_blocklog;
+	blks_per_cluster = mp->m_inode_cluster_size >> mp->m_sb.sb_blocklog;
 	if (blks_per_cluster == 0)
 		blks_per_cluster = 1;
 
@@ -710,7 +722,7 @@ pf_queuing_worker(
 		cur_irec = irec;
 
 		num_inos = XFS_INODES_PER_CHUNK;
-		while (num_inos < XFS_IALLOC_INODES(mp) && irec != NULL) {
+		while (num_inos < mp->m_ialloc_inos && irec != NULL) {
 			irec = next_ino_rec(irec);
 			num_inos += XFS_INODES_PER_CHUNK;
 		}
@@ -737,18 +749,28 @@ pf_queuing_worker(
 
 		num_inos = 0;
 		bno = XFS_AGINO_TO_AGBNO(mp, cur_irec->ino_startnum);
+		sparse = cur_irec->ir_sparse;
 
 		do {
 			struct xfs_buf_map	map;
 
 			map.bm_bn = XFS_AGB_TO_DADDR(mp, args->agno, bno);
 			map.bm_len = XFS_FSB_TO_BB(mp, blks_per_cluster);
-			pf_queue_io(args, &map, 1,
-				    (cur_irec->ino_isa_dir != 0) ?  B_DIR_INODE
-								 : B_INODE);
+
+			/*
+			 * Queue I/O for each non-sparse cluster. We can check
+			 * sparse state in cluster sized chunks as cluster size
+			 * is the min. granularity of sparse irec regions.
+			 */
+			if ((sparse & ((1 << inodes_per_cluster) - 1)) == 0)
+				pf_queue_io(args, &map, 1,
+					    (cur_irec->ino_isa_dir != 0) ?
+					     B_DIR_INODE : B_INODE);
+
 			bno += blks_per_cluster;
 			num_inos += inodes_per_cluster;
-		} while (num_inos < XFS_IALLOC_INODES(mp));
+			sparse >>= inodes_per_cluster;
+		} while (num_inos < mp->m_ialloc_inos);
 	}
 
 	pthread_mutex_lock(&args->lock);
@@ -843,9 +865,10 @@ start_inode_prefetch(
 	 */
 
 	max_queue = libxfs_bcache->c_maxcount / thread_count / 8;
-	if (XFS_INODE_CLUSTER_SIZE(mp) > mp->m_sb.sb_blocksize)
-		max_queue = max_queue * (XFS_INODE_CLUSTER_SIZE(mp) >>
-				mp->m_sb.sb_blocklog) / XFS_IALLOC_BLOCKS(mp);
+	if (mp->m_inode_cluster_size > mp->m_sb.sb_blocksize)
+		max_queue = max_queue *
+			(mp->m_inode_cluster_size >> mp->m_sb.sb_blocklog) /
+			mp->m_ialloc_blks;
 
 	sem_init(&args->ra_count, 0, max_queue);
 
